@@ -1,4 +1,4 @@
-__version__ = "0.8.10"
+__version__ = "0.10.0"
 __author__ = "refraction-ray"
 __name__ = "xalpha"
 
@@ -20,7 +20,7 @@ from xalpha.info import (
     get_fund_holdings,
 )
 from xalpha.multiple import mul, mulfix, imul, Mul, MulFix, IMul
-from xalpha.realtime import rfundinfo, review
+from xalpha.realtime import rfundinfo, review  # deprecated
 from xalpha.record import record, irecord, Record, IRecord
 from xalpha.trade import trade, itrade, Trade, ITrade
 from xalpha.universal import (
@@ -44,6 +44,8 @@ from xalpha.toolbox import (
     OverPriced,
     QDIIPredict,
     RTPredict,
+    CBCalculator,
     set_holdings,
     set_display,
 )
+import xalpha.backtest
