@@ -46,6 +46,7 @@ from xalpha.cons import (
     rget_json,
     rpost_json,
     tz_bj,
+    last_onday,
     region_trans,
     today_obj,
     _float,
@@ -502,8 +503,9 @@ selectedModule=PerformanceGraphView&selectedSubModule=Graph\
         headers={
             "sec-fetch-dest": "document",
             "sec-fetch-mode": "navigate",
-            "sec-fetch-site": "none",
+            "sec-fetch-site": "same-origin",
             "sec-fetch-user": "?1",
+            "upgrade-insecure-requests": "1",
         },
     )
     df = pd.read_excel(r.content)
@@ -808,6 +810,66 @@ def get_historical_fromycharts(code, start, end, category, metric):
     return df[["date", "close"]]
 
 
+@lru_cache()
+def get_bond_rates(rating, date=None):
+    """
+    获取各评级企业债的不同久期的预期利率
+
+    :param rating: str. eg AAA, AA-, N for 中国国债
+    :param date: %Y-%m-%d
+    :return:
+    """
+    rating = rating.strip()
+    rating_uid = {
+        "N": "2c9081e50a2f9606010a3068cae70001",
+        "AAA": "2c9081e50a2f9606010a309f4af50111",
+        "AAA-": "8a8b2ca045e879bf014607ebef677f8e",
+        "AA+": "2c908188138b62cd01139a2ee6b51e25",
+        "AA": "2c90818812b319130112c279222836c3",
+        "AA-": "8a8b2ca045e879bf014607f9982c7fc0",
+        "A+": "2c9081e91b55cc84011be40946ca0925",
+        "A": "2c9081e91e6a3313011e6d438a58000d",
+        "A-": "8a8b2ca04142df6a014148ca880f3046",
+    }
+    # 上边字典不全，非常欢迎贡献 ：）
+    def _fetch(date):
+        r = rpost(
+            "https://yield.chinabond.com.cn/cbweb-mn/yc/searchYc?\
+xyzSelect=txy&&workTimes={date}&&dxbj=0&&qxll=0,&&yqqxN=N&&yqqxK=K&&\
+ycDefIds={uid}&&wrjxCBFlag=0&&locale=zh_CN".format(
+                uid=rating_uid.get(rating, rating), date=date
+            ),
+        )
+        return r
+
+    if not date:
+        date = dt.datetime.today().strftime("%Y-%m-%d")
+
+    r = _fetch(date)
+    while len(r.text.strip()) < 20:  # 当天没有数据，非交易日
+        date = last_onday(date).strftime("%Y-%m-%d")
+        r = _fetch(date)
+    l = r.json()[0]["seriesData"]
+    l = [t for t in l if t[1]]
+    df = pd.DataFrame(l, columns=["year", "rate"])
+    return df
+
+
+def get_bond_rates_range(rating, duration=3, freq="W-FRI", start=None, end=None):
+    l = []
+    if rating.startswith("B-"):
+        rating = rating[2:]
+    rs = rating.split(".")
+    if len(rs) > 1:
+        duration = float(rs[1])
+        rating = rs[0]
+
+    for d in pd.date_range(start, end, freq=freq):
+        df = get_bond_rates(rating, d.strftime("%Y-%m-%d"))
+        l.append([d, df[df["year"] <= duration].iloc[-1]["rate"]])
+    return pd.DataFrame(l, columns=["date", "close"])
+
+
 @data_source("jq")
 def get_macro(table, start, end, datecol="stat_year"):
     df = macro.run_query(
@@ -847,7 +909,7 @@ def _get_daily(
 
             3. 对于美国市场的股票，指数，ETF 等，直接使用其字母缩写代码即可。
 
-            4. 对于人民币中间价数据，使用 "USD/CNY" 的形式，具体可能的值可在 http://www.chinamoney.com.cn/chinese/bkccpr/ 历史数据的横栏查询
+            4. 对于人民币中间价数据，使用 "USD/CNY" 的形式，具体可能的值可在 http://www.chinamoney.com.cn/chinese/bkccpr/ 历史数据的横栏查询，注意日元需要用 100JPY/CNY.
 
             5. 对于所有可以在 cn.investing.com 网站查到的金融产品，其代码可以是该网站对应的统一代码，或者是网址部分，比如 DAX 30 的概览页面为 https://cn.investing.com/indices/germany-30，那么对应代码即为 "indices/germany-30"。也可去网页 inspect 手动查找其内部代码（一般不需要自己做，推荐直接使用网页url作为 code 变量值），手动 inspect 加粗的实时价格，其对应的网页 span class 中的 pid 的数值即为内部代码。
 
@@ -891,6 +953,8 @@ def _get_daily(
 
             25. 形如 HZ999001 HZ999005 格式的数据，代表了华证系列指数 http://www.chindices.com/indicator.html#
 
+            26. 形如 B-AA+.3 格式的数据，代表了 AA+ 企业债三年久期利率数据 (每周)
+
     :param start: str. "20200101", "2020/01/01", "2020-01-01" are all legal. The starting date of daily data.
     :param end: str. format is the same as start. The ending date of daily data.
     :param prev: Optional[int], default 365. If start is not specified, start = end-prev.
@@ -1020,6 +1084,9 @@ def _get_daily(
     elif _from == "ES":
         df = get_historical_fromesunny(code, start=start, end=end)
 
+    elif _from == "B":
+        df = get_bond_rates_range(code, start=start, end=end)
+
     elif _from == "ycharts":
         df = get_historical_fromycharts(
             code,
@@ -1114,6 +1181,12 @@ def get_xueqiu_rt(code, token="a664afb60c7036c7947578ac1a5860c4cfb6b3b5"):
     timestr = dt.datetime.fromtimestamp(r["data"]["quote"]["time"] / 1000).strftime(
         "%Y-%m-%d %H:%M:%S"
     )
+    if r["data"]["quote"].get("timestamp_ext", None):
+        time_ext = dt.datetime.fromtimestamp(
+            r["data"]["quote"]["timestamp_ext"] / 1000
+        ).strftime("%Y-%m-%d %H:%M:%S")
+    else:
+        time_ext = None
     share = r["data"]["quote"]["total_shares"]
     fshare = r["data"]["quote"]["float_shares"]
     volume = r["data"]["quote"]["volume"]
@@ -1125,6 +1198,7 @@ def get_xueqiu_rt(code, token="a664afb60c7036c7947578ac1a5860c4cfb6b3b5"):
         "currency": currency,
         "market": market,  # HK, US, CN
         "time": timestr,
+        "time_ext": time_ext,
         "totshare": share,
         "floatshare": fshare,
         "volume": volume,
@@ -1190,11 +1264,17 @@ def get_cninvesting_rt(suburl, app=False):
         if span.find("a") and span.find("a")["href"].startswith("/markets"):
             market = span.string
     market = region_trans.get(market, market)
+    time_ext = s.select("div[class~=lastUpdated]")
+    if time_ext:
+        time_ext = time_ext[0].text.strip()
+    else:
+        time_ext = None
     d = {
         "name": name,
         "current": q,
         "current_ext": q_ext,
         "time": timestr,
+        "time_ext": time_ext,
         "currency": currency,
         "percent": percent,
         "market": market,
@@ -1371,27 +1451,37 @@ def get_rt_from_ttjj(code):
     r = rget("http://fund.eastmoney.com/{code}.html".format(code=code))
     r.encoding = "utf-8"
     s = BeautifulSoup(r.text, "lxml")
-    value, date, name = (
-        float(
-            s.findAll("dd", class_="dataNums")[1]
-            .find("span", class_="ui-font-large")
-            .string
-        ),
-        str(s.findAll("dt")[1]).split("(")[1].split(")")[0][7:],
-        s.select("div[style='float: left']")[0].text.split("(")[0],
-    )
+    name = s.select("div[style='float: left']")[0].text.split("(")[0]
+    if s.findAll("dd", class_="dataNums")[1].find(
+        "span", class_="ui-font-large"
+    ):  # 非货币基金
+        value, date = (
+            float(
+                s.findAll("dd", class_="dataNums")[1]
+                .find("span", class_="ui-font-large")
+                .string
+            ),
+            str(s.findAll("dt")[1]).split("(")[1].split(")")[0][7:],
+        )
+        estimate = s.select("#gz_gsz")[0].text
+        if estimate == "--":
+            estimate = None
+        else:
+            try:
+                estimate = _float(estimate)
+            except ValueError:
+                logger.warning("unrecognized estimate netvalue %s" % estimate)
+                estimate = None
+    else:
+        value, date = (
+            s.findAll("dd", class_="dataNums")[1].text,
+            str(s.findAll("dt")[1]).split("(")[1].split(")")[0],
+        )
+        estimate = None
     status = s.select("span[class='staticCell']")[0].text.strip()
     tb = s.select("div.infoOfFund > table >tr>td")
     infol = [i.text for i in tb]
-    estimate = s.select("#gz_gsz")[0].text
-    if estimate == "--":
-        estimate = None
-    else:
-        try:
-            estimate = _float(estimate)
-        except ValueError:
-            logger.warning("unrecognized estimate netvalue %s" % estimate)
-            estimate = None
+
     return {
         "name": name,
         "time": date,
@@ -1409,6 +1499,33 @@ def get_rt_from_ttjj(code):
     # 是否有美元份额计价的基金会出问题？
 
 
+@lru_cache(2048)
+def get_fund_type(code):
+    """
+    given fund code, return unified fund category which is extracted from get_rt(code)["type"]
+
+    :param code:
+    :return: str.
+    """
+    code = code[-6:]
+    t = get_rt("F" + code)["type"]
+
+    if t in ["联接基金", "股票指数"] or t.startswith("ETF"):
+        return "指数基金"
+    elif t.startswith("QDII"):
+        return "QDII"
+    elif t.startswith("股票"):
+        return "股票基金"
+    elif t.startswith("混合"):
+        return "混合基金"
+    elif t.startswith("债券"):
+        return "债券基金"
+    elif t.startswith("货币"):
+        return "货币基金"
+    else:
+        return "其他"
+
+
 def get_rt(
     code, _from=None, double_check=False, double_check_threhold=0.005, handler=True
 ):
@@ -1604,6 +1721,10 @@ def cachedio(**ioconf):
             precached = kws.get("precached", precached)
             key = kws.get("key", code)
             key = key.replace("/", " ")
+            key_func = ioconf.get("key_func", None)
+            key_func = ioconf.get("keyfunc", key_func)
+            if key_func is not None:
+                key = key_func(key)
             defaultend = ioconf.get("defaultend", today_obj)
             defaultend = ioconf.get("default_end", defaultend)
             defaultprev = ioconf.get("defaultprev", 365)
@@ -1641,6 +1762,8 @@ def cachedio(**ioconf):
             end_str = end_obj.strftime("%Y%m%d")
             backend = ioconf.get("backend")
             backend = kws.get("backend", backend)
+            # if backend == "sql": # reserved for case insensitive database settings
+            #     key = key.lower()
             refresh = ioconf.get("refresh", False)
             refresh = kws.get("refresh", refresh)
             fetchonly = ioconf.get("fetchonly", False)
@@ -1907,10 +2030,16 @@ def ttjjcode(code):
     elif code.isdigit() and len(code) == 5:
         return "HK" + code
     elif code.isdigit() and len(code) == 6:
-        if code.startswith("1") or code.startswith("0") or code.startswith("3"):
+        if (
+            code.startswith("16")
+            or code.startswith("15")
+            or code.startswith("12")
+            or code.startswith("0")
+            or code.startswith("3")
+        ):
             # 注意这里只能对应个股，指数代码有重叠没有办法的事
             return "SZ" + code
-        elif code.startswith("5") or code.startswith("6"):
+        elif code.startswith("5") or code.startswith("6") or code.startswith("11"):
             return "SH" + code
         else:
             logger.warning("unrecognized code format %s" % code)
